/*
  Connections
  In1 on Hbridge to Pin 2 on Arduino
  In2 on Hbridge to Pin 3 on Arduino
  In3 on Hbridge to Pin 4 on Arduino
  In 4on Hbridge to Pin 5 on Arduino
  IR S to Pin 6 on Arduino
  IR - to GND on Arduino
  IR VCC to 5V on Arduino
  GND on LCD IC2 decoder to GND on Arduino
  VCC on LCD IC2 decoder to 5V on Arduino
  SDA on LCD IC2 decoder to A4 on Arduino
  SCL on LCD IC2 decoder to A5 on Arduino
*/
#include <IRLib.h>
#include <Wire.h>

int RECV_PIN = 6;

IRrecv My_Receiver(RECV_PIN);
IRdecode My_Decoder;
IRdecodeHash My_Hash_Decoder;

//Xinda2
#define RIGHT_ARROW   2246090081
#define LEFT_ARROW    3419589885
#define OK_BUTTON     629155073
#define UP_ARROW      1868133565
#define DOWN_ARROW    1474512609
#define BUTTON_0      1113853081
#define BUTTON_1      3001999165
#define BUTTON_2      211564353
#define BUTTON_3      46690913
#define BUTTON_4      373604801
#define BUTTON_5      2797147069
#define BUTTON_6      3495072293
#define BUTTON_7      32049505
#define BUTTON_8      2455591773
#define BUTTON_9      3409801085
#define STAR          2985278109
#define HASHKEY       1811778305

int A = 2;
int B = 3;
int C = 4;
int D = 5;

int BTN0 = 7;
int BTN1 = 8;
int BTN2 = 9;
int BTN3 = 10;
int BTN4 = 11;
int BTN5 = 12;

int Pin07Read ;
int Pin08Read ;
int Pin09Read ;
int Pin10Read ;
int Pin11Read ;
int Pin12Read ;


long duty = 50;
int waitMicroSeconds = 100;
int pulseCount = 25;
int speed1 = (waitMicroSeconds / pulseCount) / 100 ;



//int speed1 = (waitMicroSeconds / pulseCount) / 100 ;

//int trigger = A0; // used only for testing purposes (trigger on oscilloscope)
long unsigned mvalue  ;
int location = 0 ;
const int  track0 = 0 ;
const int  track1 = 2 ;
const int  track2 = 4 ;
const int  track3 = 6 ;
const int  track4 = 8 ;
const int  track5 = 25 ;
const int right = true;
const int left = false;

void setup() {
  pinMode(A, OUTPUT);
  pinMode(B, OUTPUT);
  pinMode(C, OUTPUT);
  pinMode(D, OUTPUT);

  pinMode(BTN0, INPUT);
  pinMode(BTN1, INPUT);
  pinMode(BTN2, INPUT);
  pinMode(BTN3, INPUT);
  pinMode(BTN4, INPUT);
  pinMode(BTN5, INPUT);

  digitalWrite(BTN0, HIGH);
  digitalWrite(BTN1, HIGH);
  digitalWrite(BTN2, HIGH);
  digitalWrite(BTN3, HIGH);
  digitalWrite(BTN4, HIGH);
  digitalWrite(BTN5, HIGH);
  
  My_Receiver.enableIRIn(); // Start the receiver
  Serial.begin(9600);
}

void one() {
  digitalWrite(A, HIGH);
  digitalWrite(B, LOW);
  digitalWrite(C, HIGH);
  digitalWrite(D, LOW);
}

void two() {
  digitalWrite(A, HIGH);
  digitalWrite(B, LOW);
  digitalWrite(C, LOW);
  digitalWrite(D, HIGH);
}

void three() {
  digitalWrite(A, LOW);
  digitalWrite(B, HIGH);
  digitalWrite(C, LOW);
  digitalWrite(D, HIGH);
}

void four() {
  digitalWrite(A, LOW);
  digitalWrite(B, HIGH);
  digitalWrite(C, HIGH);
  digitalWrite(D, LOW);
}


void oneB() {
  digitalWrite(A, HIGH);
  digitalWrite(B, LOW);
  digitalWrite(C, LOW);
  digitalWrite(D, LOW);
}

void twoB() {
  digitalWrite(A, LOW);
  digitalWrite(B, LOW);
  digitalWrite(C, LOW);
  digitalWrite(D, HIGH);
}

void threeB() {
  digitalWrite(A, LOW);
  digitalWrite(B, HIGH);
  digitalWrite(C, LOW);
  digitalWrite(D, LOW);
}

void fourB() {
  digitalWrite(A, LOW);
  digitalWrite(B, LOW);
  digitalWrite(C, HIGH);
  digitalWrite(D, LOW);
}


// main routine to microstep
void doStep(int st) {

  long dt1 = waitMicroSeconds * duty / 100;
  long dt2 = waitMicroSeconds * (100 - duty) / 100;

  for (int j = 0; j < pulseCount; j++) {
    switch (st) {
      case 1: one(); break;
      case 2: two(); break;
      case 3: three(); break;
      case 4: four(); break;
      case 11: oneB(); break;
      case 12: twoB(); break;
      case 13: threeB(); break;
      case 14: fourB(); break;

      case 21: one(); break;
      case 22: two(); break;
      case 23: three(); break;
      case 24: four(); break;
      case 31: oneB(); break;
      case 32: twoB(); break;
      case 33: threeB(); break;
      case 34: fourB(); break;

    }

    delayMicroseconds(dt1);

    switch (st) {
      case 1: one(); break;
      case 2: two(); break;
      case 3: three(); break;
      case 4: four(); break;
      case 11: oneB(); break;
      case 12: twoB(); break;
      case 13: threeB(); break;
      case 14: fourB(); break;

      case 21: oneB(); break;
      case 22: twoB(); break;
      case 23: threeB(); break;
      case 24: fourB(); break;
      case 31: two(); break;
      case 32: three(); break;
      case 33: four(); break;
      case 34: one(); break;
    }
    delayMicroseconds(dt2);

  }
}

// disable motor
void motorOff() {
  /* Important note:
       Turning off the motor will make it go into a 'rest' state.
       When using microsteps (or even full steps), this may not be the last active step.
       So using this routine may change the position of the motor a bit.
  */

  digitalWrite(A, LOW);
  digitalWrite(B, LOW);
  digitalWrite(C, LOW);
  digitalWrite(D, LOW);
}


// microstepping >16 steps :
void doMoreSteps(int cnt, boolean forwards) {
  const int list1[] = {1, 11, 2, 12, 3, 13, 4, 14};
  const int list2[] = {21, 31, 22, 32, 23, 33, 24, 34};

  for (int i = 0; i < cnt; i++) {

    duty = 50;
    if (forwards)
    {
      for (int j = 0; j <= 7; j++) {
        doStep(list1[j]);
        doSteps(list2[j], forwards);
      }
    }
    else
    {
      for (int j = 7; j >= 0; j--) {
        doSteps(list2[j], forwards);
        doStep(list1[j]);
      }
    }

  }
}

// this routine handles >16 microsteps
// uncomment sections to choose # steps
void doSteps(int st, boolean forwards) {



  // *********************** 96 steps
  if (forwards) {
    for (int i = 9; i >= 1; i--) {
      duty = 10 * i;
      doStep(st);
    }
  }
  else {
    for (int i = 1; i <= 9; i++) {
      duty = 10 * i;
      doStep(st);
    }
  }

}


// the loop routine runs over and over again forever:
void loop() {

  // uncomment this to disable motor
  //  motorOff();return;
  digitalWrite(BTN0, HIGH);
  digitalWrite(BTN1, HIGH);
  digitalWrite(BTN2, HIGH);
  digitalWrite(BTN3, HIGH);
  digitalWrite(BTN4, HIGH);
  digitalWrite(BTN5, HIGH);


  /* control the speed of the motor with a waitMicroseconds/pulseCount pair
     e.g. waitMicroSeconds / pulseCount
          500 / 5 --> one step takes 2500 microseconds
          50 / 50 --> one step also takes 2500 microseconds
          but in second pair the fequency of the wave is 20kHz, not audible..
          in 500 / 5 you may hear a high tone in the motor in microsteps
          some motors may not respond well on higher frequencies
     note: these parameters also control speed in normal stepping (full or half) although there's no pulses in those cases,
              setting pulseCount = 1 might be more readable
  */
  waitMicroSeconds = 500;
  pulseCount = 50 ;
  speed1 = (waitMicroSeconds / pulseCount) / 100 ;
  mvalue = 0 ;
  /*  uncomment one of the next routines to choose stepping mode
     two parameters:
      1. number of steps
      2. forwards = true --> move forwards   /  forwards = left --> move backwards

    note: there's no speed correction for different modes, so do4Steps makes the motor go twice as fast as do8Steps and so on...
          use waitMicroseconds and pulseCount to adapt speed when changing modes
  */
  if (My_Receiver.GetResults(&My_Decoder)) {//Puts results in My_Decoder
    My_Hash_Decoder.copyBuf(&My_Decoder);//copy the results to the hash decoder
    My_Decoder.decode();
    My_Hash_Decoder.decode();
    mvalue = (My_Hash_Decoder.hash) ;
    //Serial.println(My_Hash_Decoder.hash, DEC); // Do something interesting with this value
    My_Receiver.resume();
  }
  Pin07Read = digitalRead(BTN0) ;
  Pin08Read = digitalRead(BTN1) ;
  Pin09Read = digitalRead(BTN2) ;
  Pin10Read = digitalRead(BTN3) ;
  Pin11Read = digitalRead(BTN4) ;
  Pin12Read = digitalRead(BTN5) ;

  if (mvalue == OK_BUTTON)  {
    doMoreSteps(0, right);
    motorOff();
    location = 0 ;
  }
  if (mvalue == RIGHT_ARROW ) {
    doMoreSteps(1, right);
    motorOff();
    mvalue = 0;
    location = location + 1 ;
  }
  if (mvalue == LEFT_ARROW  ) {
    doMoreSteps(1, left);
    motorOff();
    mvalue = 0 ;
    location = location - 1 ;
  }
  if (mvalue == BUTTON_0 || Pin07Read == LOW) {
    switch (location) {
      case track0 :
        // do noth
        break ;
      case track1 :
        doMoreSteps(track1 - track0, left) ;
        break ;
      case track2 :
        doMoreSteps(track2 - track0, left) ;
        break ;
      case track3 :
        doMoreSteps(track3 - track0, left) ;
        break ;
      case track4 :
        doMoreSteps(track4 - track0, left) ;
        break ;
      case track5 :
        doMoreSteps(track5 - track0, left) ;
        break ;
    }
    location = track0 ;
  }
  if (mvalue == BUTTON_1 || Pin08Read == LOW) {
    switch (location) {
      case track0 :
        doMoreSteps(track1, right) ;
        break ;
      case track1 :
        // do noth
        break ;
      case track2 :
        doMoreSteps(track2 - track1, left) ;
        break ;
      case track3 :
        doMoreSteps(track3 - track1, left) ;
        break ;
      case track4 :
        doMoreSteps(track4 - track1, left) ;
        break ;
      case track5 :
        doMoreSteps(track5 - track1, left) ;
        break ;
    }
    location = track1 ;
  }
  if (mvalue == BUTTON_2 || Pin09Read == LOW) {
    switch (location) {
      case track0 :
        doMoreSteps(track2, right) ;
        break ;
      case track1 :
        doMoreSteps(track2 - track1, right) ;
        break ;
      case track2 :
        // do noth
        break ;
      case track3 :
        doMoreSteps(track3 - track2, left) ;
        break ;
      case track4 :
        doMoreSteps(track4 - track2, left) ;
        break ;
      case track5 :
        doMoreSteps(track5 - track2, left) ;
        break ;
    }
    location = track2 ;
  }
  if (mvalue == BUTTON_3 || Pin10Read == LOW) {
    switch (location) {
      case track0 :
        doMoreSteps(track3, right) ;
        break ;
      case track1 :
        doMoreSteps(track3 - track1, right) ;
        break ;
      case track2 :
        doMoreSteps(track3 - track2, right) ;
        break ;
      case track3 :
        // do noth
        break ;
      case track4 :
        doMoreSteps(track4 - track3, left) ;
        break ;
      case track5 :
        doMoreSteps(track5 - track3, left) ;
        break ;
    }
    location = track3 ;
  }
  if (mvalue == BUTTON_4 || Pin11Read == LOW) {
    switch (location) {
      case track0 :
        doMoreSteps(track4, right) ;
        break ;
      case track1 :
        doMoreSteps(track4 - track1, right) ;
        break ;
      case track2 :
        doMoreSteps(track4 - track2, right) ;
        break ;
      case track3 :
        doMoreSteps(track4 - track3, right) ;
        break ;
      case track4 :
        // do noth
        break ;
      case track5 :
        doMoreSteps(track5 - track3, left) ;
        break ;
    }
    location = track4 ;
  }
  if (mvalue == BUTTON_5 || Pin12Read == LOW) {
    switch (location) {
      case track0 :
        doMoreSteps(track5, right) ;
        break ;
      case track1 :
        doMoreSteps(track5 - track1, right) ;
        break ;
      case track2 :
        doMoreSteps(track5 - track2, right) ;
        break ;
      case track3 :
        doMoreSteps(track5 - track3, right) ;
        break ;
      case track4 :
        doMoreSteps(track5 - track4, right) ;
        break ;
      case track5 :
        location = track5 ;
        // do noth
        break ;
    }
    location = track0 ;
  }
  motorOff(); // give it a rest (see note in motorOff about losing steps)
}
